@@ -1,6 +1,6 @@
 /* FluidSynth DSSI software synthesizer plugin
  *
- * Copyright (C) 2004-2005 Sean Bolton and others.
+ * Copyright (C) 2004-2008 Sean Bolton and others.
  *
  * Portions of this file may have come from Peter Hanappe's
  * Fluidsynth, copyright (C) 2003 Peter Hanappe and others.
@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
- * MA 02111-1307, USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  */
 
 #define _BSD_SOURCE    1
@@ -73,7 +73,18 @@ fsd_run_multiple_synths(unsigned long instance_count, LADSPA_Handle *handles,
                         unsigned long sample_count, snd_seq_event_t **events,
                         unsigned long *event_counts);
 
-/* ---- mutual exclusion ---- */
+/* ---- FluidSynth helper functions ---- */
+
+/*
+ * fsd_chan_all_voices_off
+ *
+ * turn off all voices on channel immediately
+ */
+static inline void
+fsd_chan_all_voices_off(int channel)
+{
+    fluid_synth_cc(fsd_synth.fluid_synth, channel, 0x78, 0);  /* 0x78 = MIDI 'all sound off' control change */
+}
 
 /*
  * fsd_all_voices_off
@@ -86,10 +97,12 @@ fsd_all_voices_off(void)
     int i;
 
     for (i = 0; i < FSD_CHANNEL_COUNT; i++) {
-        fluid_synth_all_sounds_off(fsd_synth.fluid_synth, i);
+        fsd_chan_all_voices_off(i);
     }
 }
 
+/* ---- mutual exclusion ---- */
+
 static inline int
 fsd_mutex_trylock(void)
 {
@@ -416,7 +429,7 @@ fsd_deactivate(LADSPA_Handle handle)
     fsd_instance_t *instance = (fsd_instance_t *)handle;
 
     /* stop all voices on channel immediately */
-    fluid_synth_all_sounds_off(fsd_synth.fluid_synth, instance->channel);
+    fsd_chan_all_voices_off(instance->channel);
 }
 
 /*
@@ -440,8 +453,14 @@ fsd_cleanup(LADSPA_Handle handle)
         fsd_synth.channel_map[instance->channel] = NULL;
 
 #ifndef NWRITE_FLOAT_WORKS_CORRECTLY
-        if (instance->tmpbuf_l) free(instance->tmpbuf_l);
-        if (instance->tmpbuf_r) free(instance->tmpbuf_r);
+        if (instance->tmpbuf_l) {
+            free(instance->tmpbuf_l);
+            instance->tmpbuf_l = NULL;
+        }
+        if (instance->tmpbuf_r) {
+            free(instance->tmpbuf_r);
+            instance->tmpbuf_r = NULL;
+        }
 #endif
     }
 
@@ -809,11 +828,10 @@ fsd_run_multiple_synths(unsigned long instance_count, LADSPA_Handle *handles,
     }
 
 #ifdef NWRITE_FLOAT_WORKS_CORRECTLY
-    /* If fluid_synth_nwrite_float() worked correctly for block lengths less
-     * than FLUID_BUFSIZE (64), we could just do the following, and save a
-     * copy. But it doesn't in fluidsynth versions up to at least 1.0.5.
-     * (Note that if the block length is _always_ 32, the bug is not triggered,
-     * luckily, simply because 64 - 32 = 32 (see the code)). */
+    /* fluid_synth_nwrite_float() works correctly in FluidSynth beginning
+     * with CVS as of 2005/6/7 (and I would assume releases > 1.0.5).
+     * So here we can just have it write our output buffers directly,
+     * without the extra copy. */
 
     next_pending_event_tick = 0;
     while (samples_done < sample_count) {
@@ -858,10 +876,13 @@ fsd_run_multiple_synths(unsigned long instance_count, LADSPA_Handle *handles,
 
 #else /* fluid_synth_nwrite_float() doesn't work correctly */
 
-    /* Because of the fluid_synth_nwrite_float() bug, we have to always call
-     * it with block lengths that are multiples of FLUID_BUFSIZE (64),
-     * buffering any odd block remains for the next run_multiple_synths()
-     * call ourself.  Ick.  */
+    /* Because fluid_synth_nwrite_float() doesn't work correctly in FluidSynth
+     * versions <= 1.0.5 (including CVS before 2005/6/7) for block lengths less
+     * than FLUID_BUFSIZE (64), we have to always call it with block lengths
+     * that are multiples of 64, buffering any odd block remains for the next
+     * run_multiple_synths() call ourself.
+     * (Note that if the block length is _always_ 32, the bug is not triggered,
+     * luckily, simply because 64 - 32 = 32 (see the code)). */
 
     /* First, if there is any data remaining from a previous render burst,
      * copy it from our temporary buffers. */
@@ -908,7 +929,11 @@ fsd_run_multiple_synths(unsigned long instance_count, LADSPA_Handle *handles,
         /* render the burst */
         burst_size = next_pending_event_tick - samples_done;
         burst_size = (burst_size + fsd_synth.fluid_bufsize - 1) &
-                         ~(fsd_synth.fluid_bufsize - 1);
+                         ~(fsd_synth.fluid_bufsize - 1);   /* round up to nearest whole bufsize */
+        if (burst_size > sample_count - samples_done)      /* don't overshoot sample_count */
+            burst_size -= fsd_synth.fluid_bufsize;
+        if (burst_size == 0)                               /* leave loop if too close to end */
+            break;
         if (burst_size > FSD_MAX_BURST_SIZE)
             burst_size = FSD_MAX_BURST_SIZE;
         
@@ -949,20 +974,12 @@ fsd_run_multiple_synths(unsigned long instance_count, LADSPA_Handle *handles,
     if (samples_done < sample_count) {
         unsigned long samples_remaining = (sample_count - samples_done);
 
-        /* silence temporary buffers */
-        for (i = 0; i < instance_count; i++) {
-            memset(instances[i]->tmpbuf_l, 0,
-                   fsd_synth.fluid_bufsize * sizeof(LADSPA_Data));
-            memset(instances[i]->tmpbuf_r, 0,
-                   fsd_synth.fluid_bufsize * sizeof(LADSPA_Data));
-        }
-
         /* render one burst */
         for (i = 0; i < FSD_CHANNEL_COUNT; i++) {
             if (fsd_synth.channel_map[i]) {
                 fsd_instance_t *instance = fsd_synth.channel_map[i];
-                l_outputs[instance->channel] = instance->tmpbuf_l + samples_done;
-                r_outputs[instance->channel] = instance->tmpbuf_r + samples_done;
+                l_outputs[instance->channel] = instance->tmpbuf_l;
+                r_outputs[instance->channel] = instance->tmpbuf_r;
             }
         }
 
@@ -1024,7 +1041,11 @@ const DSSI_Descriptor *dssi_descriptor(unsigned long index)
     }
 }
 
+#ifdef __GNUC__
+__attribute__((constructor)) void init()
+#else
 void _init()
+#endif
 {
     int i;
     char **port_names;
@@ -1098,7 +1119,11 @@ void _init()
     }
 }
 
+#ifdef __GNUC__
+__attribute__((destructor)) void fini()
+#else
 void _fini()
+#endif
 {
     if (fsd_LADSPA_descriptor) {
         free((LADSPA_PortDescriptor *) fsd_LADSPA_descriptor->PortDescriptors);
